@@ -8,21 +8,36 @@ our @EXPORT = ('bash');
 our @EXPORT_OK = (@EXPORT, qw< pwd head tail >);
 our %EXPORT_TAGS = ( all => \@EXPORT_OK );
 
-our $VERSION = '0.03_01'; # TRIAL VERSION
+our $VERSION = '0.03_02'; # TRIAL VERSION
 
 
 use Carp;
 use Contextual::Return;
+use List::Util qw< any >;
 use Scalar::Util qw< blessed >;
 use IPC::System::Simple qw< run capture EXIT_ANY $EXITVAL >;
 
 
+my @AUTOQUOTE =
+(
+	sub {	ref shift eq 'Regexp'						},
+	sub {	blessed $_[0] and $_[0]->can('basename')	},
+);
+
+sub _should_quote ()
+{
+	my $arg = $_;
+	local $_;
+	return 1 if any { $_->($arg) } @AUTOQUOTE;
+	return 0;
+}
+
 sub _process_bash_arg ()
 {
 	# incoming arg is in $_
 	my $arg = $_;				# make a copy
 	croak("Use of uninitialized argument to bash") unless defined $arg;
-	if (ref $arg eq 'Regexp' or blessed $arg and $arg->can('basename'))
+	if (_should_quote)
 	{
 		$arg = "$arg";			# stringify
 		$arg =~ s/'/'\\''/g;	# handle internal single quotes
@@ -42,7 +57,7 @@ sub bash (@)
 		my $arg = shift;
 		if (ref $arg)
 		{
-			die("bash: multiple capture specifications") if $capture;
+			croak("bash: multiple capture specifications") if $capture;
 			$capture = $$arg;
 		}
 		elsif ($arg eq '-e')
@@ -56,6 +71,10 @@ sub bash (@)
 	}
 	croak("Not enough arguments for bash") unless @_;
 
+	my $filter;
+	$filter = pop if ref $_[-1] eq 'CODE';
+	croak("bash: multiple output redirects") if $capture and $filter;
+
 	my @cmd = 'bash';
 	push @cmd, @opts;
 	push @cmd, '-c';
@@ -68,7 +87,7 @@ sub bash (@)
 		my $IFS = $ENV{IFS};
 		$IFS = " \t\n" unless defined $IFS;
 
-		my $output = capture [0..125], qw< bash -c >, $bash_cmd;
+		my $output = capture $exit_codes, qw< bash -c >, $bash_cmd;
 		if ($capture eq 'string')
 		{
 			return $output;
@@ -88,6 +107,35 @@ sub bash (@)
 			die("bash: unrecognized capture specification [$capture]");
 		}
 	}
+	elsif ($filter)
+	{
+		$cmd[-1] =~ s/\s*(?<!\|)\|(&)?\s*$// or croak("bash: cannot filter without redirect");
+		$cmd[-1] .= ' 2>&1' if $1;
+
+		# This is pretty much straight out of `man perlipc`.
+		local *CHILD;
+		my $pid = open(CHILD, "-|");
+		defined($pid) or die("bash: can't fork [$!]");
+
+		if ($pid)				# parent
+		{
+			local $_;
+			while (<CHILD>)
+			{
+				$filter->($_);
+			}
+			unless (close(CHILD))
+			{
+				# You know how IPC::System::Simple says that `_process_child_error` is not intended
+				# to be called directly?  Yeah, well, the alternatives are worse ...
+				IPC::System::Simple::_process_child_error($?, 'bash', $exit_codes);
+			}
+		}
+		else					# child
+		{
+			exec(@cmd) or die("bash: can't exec program [$!]");
+		}
+	}
 	else
 	{
 		run $exit_codes, @cmd;
@@ -139,7 +187,7 @@ PerlX::bash - tighter integration between Perl and bash
 
 =head1 VERSION
 
-This document describes version 0.03_01 of PerlX::bash.
+This document describes version 0.03_02 of PerlX::bash.
 
 =head1 SYNOPSIS
 
@@ -197,7 +245,7 @@ Buddy Burden <barefootcoder@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2015-2017 by Buddy Burden.
+This software is Copyright (c) 2015-2019 by Buddy Burden.
 
 This is free software, licensed under:
 
